
// --------------------------------------
    
// lambda1 
// 2009/Apr/17

// random correlated diffusion in a 2d random landscape
  
// --------------------------------------


#include "../../CMLib/CompleteLib.h"

double len,wDepth,wWidth,wStiff,wDist,tinyEps,dT,t1,friction;
double rSigma,rTau,rGamma,kConf,xBoundary;
int NWELL,TMX;
bool STORE_ALL;

double *wDep,*wWid;
vec2D *wPos,*rFrc;
CorGauPro fluctX, fluctY;


// --------------------------------------

void DefParameters(void) {


	// simulation control

	TMX=1000;               // nr. of simulation time steps (100000 is good)
	RNDINI=-9730375;        // seed for random number generator
	dT=0.1;                 // time step of force
	STORE_ALL=true;         // store everything ?   


	// potential landscape

	NWELL=10;    // nr. of potential wells
	len=20.0; // linear size of landscape

	// well properties
	wDepth=10.0; // depth of gaussian wells
	wWidth=1.0;  // width of gaussian wells
	
	// parabolic well case
	wStiff=0.0e-2; // stiffness of parabolic well

	// double well case
	wDist=2.5;  // distance between wells

	// outer confinement
	kConf=10.0;
	xBoundary=len/2.0;

	// viscous background
	friction=1.0;


	// selfgenerated force

	rGamma=20.0;   // effective white noise strengh
	rTau=1.0;     // correlation time of random force
	rSigma=sqrt(rGamma/(2.0*rTau));   // standard deviation of random force
	

	// derived

	tinyEps=wWidth*1.0e-10; // for numerical derivatives
  t1=dT*(double)TMX;      // simulation stop time

}

// --------------------------------------

double uSingle(vec2D dr, int W) {
	double s=abs(dr)/wWid[W];
	return (-wDep[W]*exp(-0.5*s*s));  // !!!!!!!!!!!!!!!!!!
	//return (0.5*wStiff*dx*dx);
}

// --------------------------------------

void AllocArrays(void) {

	wDep=NRvector<double>(1,NWELL);
	wWid=NRvector<double>(1,NWELL);

	wPos=NRvector<vec2D>(1,NWELL);
	rFrc=NRvector<vec2D>(0,TMX);
}

// --------------------------------------

vec2D fSingle(vec2D pos, int W) {

	// x-component of gradient
	vec2D dx=vec2D(tinyEps,0.0);
	double ux2=uSingle(pos+dx,W);
	double ux1=uSingle(pos,W);
	double fx=(-(ux2-ux1)/tinyEps);

  // y-component of gradient
	vec2D dy=vec2D(0.0,tinyEps);
	double uy2=uSingle(pos+dy,W);
	double uy1=uSingle(pos,W);
	double fy=(-(uy2-uy1)/tinyEps);

	return vec2D(fx,fy);
}


// --------------------------------------

void GenLandscape(void) {


	if (NWELL==1) {

    wPos[1]=vec2D(0.0,0.0);
		wDep[1]=wDepth;
		wWid[1]=wWidth;

	} 
	

	if (NWELL==2) {

    wPos[1]=vec2D(-wDist/2.0,0.0);
    wPos[2]=vec2D(+wDist/2.0,0.0);

		wDep[1]=wDepth;
		wDep[2]=wDepth;

		wWid[1]=wWidth;
		wWid[2]=wWidth;

	}


	if (NWELL>2) {

	  int W;
	  for (W=1;W<=NWELL;W++) {
		  wPos[W]=vec2D( boxRnd(&RNDINI,-len/2.0,len/2.0) , boxRnd(&RNDINI,-len/2.0,len/2.0) );
		  wDep[W]=wDepth;
		  wWid[W]=wWidth;
		}//W

	}
}

// --------------------------------------

void GenForce(void) {

	fluctX.reset(&RNDINI, 0.0, rSigma, rTau, dT);
  fluctY.reset(&RNDINI, 0.0, rSigma, rTau, dT);

	int T;

	for (T=0;T<=TMX;T++) {
    rFrc[T]=vec2D( fluctX.nextVal(), fluctY.nextVal() );
	}//T

}

// --------------------------------------

double potExt(vec2D pos) {
	int W;
	double sum=0.0;
  for (W=1;W<=NWELL;W++) {
	  sum+=uSingle(pos-wPos[W],W);
	}//W
	return sum;
}

// --------------------------------------

vec2D frcExt(vec2D pos) {

	// potential wells
	int W;
	vec2D sum=0.0;
  for (W=1;W<=NWELL;W++) {
	  sum+=fSingle(pos-wPos[W],W);
	}//W

	// outer confinement

	double x=real(pos);
	double y=imag(pos);
	if (x> xBoundary) {sum+=vec2D(kConf*(xBoundary-x),0.0);}
	if (x<-xBoundary) {sum+=vec2D(kConf*(-xBoundary-x),0.0);}
	if (y> xBoundary) {sum+=vec2D(0.0,kConf*(xBoundary-y));}
	if (y<-xBoundary) {sum+=vec2D(0.0,kConf*(-xBoundary-y));}

	return sum;
}

// --------------------------------------

vec2D frcInt(double t) {
	int T=t/dT;
	if ((T>=0)&&(T<=TMX)) {return rFrc[T];}
	else {return vec2D(0.0,0.0);}
}

// --------------------------------------

void DGLDef(double t, double yt[], double dydt[])
{

	// dynamic variables
	vec2D position=vec2D(yt[1],yt[2]); 
  vec2D force=frcExt(position)+frcInt(t); 

	// overdamped equations of motion
	// ---------------------------
  vec2D velocity=force/friction;
	// ---------------------------

	// external arrays
	dydt[1]=real(velocity);
	dydt[2]=imag(velocity);
	
}

// --------------------------------------

void DGLIni(double y0[])
{
	y0[1]=1.0;
	y0[2]=1.0;
}

// --------------------------------------

void ComputeTrajectory(void) {

	SampledVecFu partPos(0.0,t1,TMX,2);
  partPos.solveDE(DGLDef,DGLIni);

	int T;
	DRecXY traj;
	for (T=0;T<=TMX;T++) {
		traj.record( partPos.yy[1][T] , partPos.yy[2][T] );
	}
	traj.store("data/traj.dat");
}

// --------------------------------------


// --------------------------------------


void StoreData(void) {

	if (STORE_ALL) {

		
    // plot landscape
    int NMX=20;
    double **land;
		land=NRmatrix<double>(0,NMX,0,NMX);
		double x0=-len/2.0;
		double dx=len/((double)(NMX+1));
		double y0=-len/2.0;
		double dy=len/((double)(NMX+1));
		double x,y;
		int X,Y;
		double pMin=1.0e50;
		double pMax=-1.0e50;
		for (X=0;X<=NMX;X++) {
		for (Y=0;Y<=NMX;Y++) {
			x=x0+(dx/2.0)+(dx*(double)X);
      y=y0+(dy/2.0)+(dy*(double)Y);
			land[X][Y]=potExt(vec2D(x,y)); 
			if (land[X][Y]<pMin) {pMin=land[X][Y];}
			if (land[X][Y]>pMax) {pMax=land[X][Y];}
		}//Y
		}//X
    //printf("pMin=%.2f, pMax=%.2f\n",pMin,pMax);
		Store2D(land,0,NMX,0,NMX,"data/land2D.dat");


		// plot obstacle density
		DRecXY points;
		double value;
		double maxDens=5.0;
		int P,PMX;
		for (X=0;X<=NMX;X++) {
		for (Y=0;Y<=NMX;Y++) {
			value=(land[X][Y]-pMin)/(pMax-pMin); 
			if (value>0.7) {
      PMX=(int)(maxDens*value); //printf("X=%d, Y=%d, PMX=%d\n",X,Y,PMX);
			for (P=0;P<=PMX;P++) {
				x=x0+(dx/2.0)+(dx*(double)X);
        y=y0+(dy/2.0)+(dy*(double)Y);
        points.record( x+boxRnd(&RNDINI,-dx/2.0,dx/2.0) , y+boxRnd(&RNDINI,-dy/2.0,dy/2.0));
			}//P
			}
		}//Y
		}//X
		points.store("data/obstacles.dat");

	}//ifSTORE_ALL

}



// --------------------------------------


// ____________________________________________________________________________________________


void main(void) {

	printf("-------- start -----------\n");

  ClearDataFolder();

	DefParameters();
	AllocArrays();
	GenLandscape();
	GenForce(); 
	StoreData();
  ComputeTrajectory(); 

  printf("-------- stop  -----------\n");

}

// ____________________________________________________________________________________________




